@@ -1,6 +1,7 @@
-/* $Id: tap-provider.c 2033 2006-09-13 23:36:30Z benny $ */
+/* vi:set et ai sw=2 sts=2 ts=2: */
 /*-
  * Copyright (c) 2006 Benedikt Meurer <benny@xfce.org>
+ * Copyright (c) 2011 Jannis Pohlmann <jannis@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -9,13 +10,13 @@
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+ * GNU Library General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Library General 
+ * Public License along with this library; if not, write to the 
+ * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,7 +27,7 @@
 #include <unistd.h>
 #endif
 
-#include <thunar-vfs/thunar-vfs.h>
+#include <libxfce4util/libxfce4util.h>
 
 #include <thunar-archive-plugin/tap-backend.h>
 #include <thunar-archive-plugin/tap-provider.h>
@@ -40,9 +41,7 @@
 
 
 
-static void   tap_provider_class_init           (TapProviderClass         *klass);
 static void   tap_provider_menu_provider_init   (ThunarxMenuProviderIface *iface);
-static void   tap_provider_init                 (TapProvider              *tap_provider);
 static void   tap_provider_finalize             (GObject                  *object);
 static GList *tap_provider_get_file_actions     (ThunarxMenuProvider      *menu_provider,
                                                  GtkWidget                *window,
@@ -85,18 +84,19 @@ struct _TapProvider
   GtkIconFactory *icon_factory;
 #endif
 
-  /* child watch support for the last spawn command,
-   * which allows us to refresh the folder contents
-   * after the command terminates (i.e. the archive
-   * is created).
+  /* child watch support for the last spawn command, which allowed us to refresh 
+   * the folder contents after the command had terminated with ThunarVFS (i.e. 
+   * when the archive had been created). This no longer works with GIO but 
+   * we still use the watch to close the PID properly.
    */
-  gchar          *child_watch_path;
   gint            child_watch_id;
 };
 
 
 
 static const gchar TAP_MIME_TYPES[][34] = {
+  "application/x-7z-compressed",
+  "application/x-7z-compressed-tar",
   "application/x-ar",
   "application/x-arj",
   "application/x-bzip",
@@ -108,9 +108,13 @@ static const gchar TAP_MIME_TYPES[][34] = {
   "application/x-gzip",
   "application/x-lha",
   "application/x-lhz",
+  "application/x-lzma",
+  "application/x-lzma-compressed-tar",
   "application/x-rar",
   "application/x-rar-compressed",
   "application/x-tar",
+  "application/x-xz",
+  "application/x-xz-compressed-tar",
   "application/x-zip",
   "application/x-zip-compressed",
   "application/zip",
@@ -121,6 +125,7 @@ static const gchar TAP_MIME_TYPES[][34] = {
   "application/x-lzop",
   "application/x-zoo",
   "application/x-cd-image",
+  "application/x-7z-compressed",
 };
 
 static GQuark tap_action_files_quark;
@@ -220,7 +225,6 @@ tap_provider_finalize (GObject *object)
     {
       /* reset the callback function to g_spawn_close_pid() so the plugin can be
        * safely unloaded and the child will still not become a zombie afterwards.
-       * This also resets the child_watch_id and child_watch_path properties.
        */
       source = g_main_context_find_source_by_id (NULL, tap_provider->child_watch_id);
       g_source_set_callback (source, (GSourceFunc) g_spawn_close_pid, NULL, NULL);
@@ -240,7 +244,7 @@ tap_provider_finalize (GObject *object)
 static gboolean
 tap_is_archive (ThunarxFileInfo *file_info)
 {
-  gint n;
+  guint n;
 
   for (n = 0; n < G_N_ELEMENTS (TAP_MIME_TYPES); ++n)
     if (thunarx_file_info_has_mime_type (file_info, TAP_MIME_TYPES[n]))
@@ -419,8 +423,7 @@ tap_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
                                GtkWidget           *window,
                                GList               *files)
 {
-  ThunarVfsPathScheme scheme;
-  ThunarVfsInfo      *info;
+  gchar              *scheme;
   TapProvider        *tap_provider = TAP_PROVIDER (menu_provider);
   GtkAction          *action;
   GClosure           *closure;
@@ -434,13 +437,15 @@ tap_provider_get_file_actions (ThunarxMenuProvider *menu_provider,
   for (lp = files; lp != NULL; lp = lp->next, ++n_files)
     {
       /* check if the file is a local file */
-      info = thunarx_file_info_get_vfs_info (lp->data);
-      scheme = thunar_vfs_path_get_scheme (info->path);
-      thunar_vfs_info_unref (info);
+      scheme = thunarx_file_info_get_uri_scheme (lp->data);
 
       /* unable to handle non-local files */
-      if (G_UNLIKELY (scheme != THUNAR_VFS_PATH_SCHEME_FILE))
-        return NULL;
+      if (G_UNLIKELY (strcmp (scheme, "file")))
+        {
+          g_free (scheme);
+          return NULL;
+        }
+      g_free (scheme);
 
       /* check if this file is a supported archive */
       if (all_archives && !tap_is_archive (lp->data))
@@ -547,8 +552,7 @@ tap_provider_get_dnd_actions (ThunarxMenuProvider *menu_provider,
                               ThunarxFileInfo     *folder,
                               GList               *files)
 {
-  ThunarVfsPathScheme scheme;
-  ThunarVfsInfo      *info;
+  gchar              *scheme;
   TapProvider        *tap_provider = TAP_PROVIDER (menu_provider);
   GtkAction          *action;
   GClosure           *closure;
@@ -556,25 +560,29 @@ tap_provider_get_dnd_actions (ThunarxMenuProvider *menu_provider,
   gint                n_files = 0;
 
   /* check if the folder is a local folder */
-  info = thunarx_file_info_get_vfs_info (folder);
-  scheme = thunar_vfs_path_get_scheme (info->path);
-  thunar_vfs_info_unref (info);
+  scheme = thunarx_file_info_get_uri_scheme (folder);
 
   /* unable to extract to non-local folders */
-  if (G_UNLIKELY (scheme != THUNAR_VFS_PATH_SCHEME_FILE))
-    return NULL;
+  if (G_UNLIKELY (strcmp (scheme, "file")))
+    {
+      g_free (scheme);
+      return NULL;
+    }
+  g_free (scheme);
 
   /* check all supplied files */
   for (lp = files; lp != NULL; lp = lp->next, ++n_files)
     {
       /* check if the file is a local file */
-      info = thunarx_file_info_get_vfs_info (lp->data);
-      scheme = thunar_vfs_path_get_scheme (info->path);
-      thunar_vfs_info_unref (info);
+      scheme = thunarx_file_info_get_uri_scheme (lp->data);
 
       /* unable to handle non-local files */
-      if (G_UNLIKELY (scheme != THUNAR_VFS_PATH_SCHEME_FILE))
-        return NULL;
+      if (G_UNLIKELY (strcmp (scheme, "file")))
+        {
+          g_free (scheme);
+          return NULL;
+        }
+      g_free (scheme);
 
       /* check if this file is a supported archive */
       if (G_LIKELY (!tap_is_archive (lp->data)))
@@ -648,10 +656,6 @@ tap_provider_execute (TapProvider *tap_provider,
       /* schedule the new child watch */
       tap_provider->child_watch_id = g_child_watch_add_full (G_PRIORITY_LOW, pid, tap_provider_child_watch,
                                                              tap_provider, tap_provider_child_watch_destroy);
-
-      
-      /* remember the working directory for the child watch */
-      tap_provider->child_watch_path = g_strdup (folder);
     }
   else if (error != NULL)
     {
@@ -676,29 +680,8 @@ tap_provider_child_watch (GPid     pid,
                           gint     status,
                           gpointer user_data)
 {
-  ThunarVfsMonitor  *monitor;
-  ThunarVfsPath     *path;
-  TapProvider       *tap_provider = TAP_PROVIDER (user_data);
-
   GDK_THREADS_ENTER ();
 
-  /* verify that we still have a valid child_watch_path */
-  if (G_LIKELY (tap_provider->child_watch_path != NULL))
-    {
-      /* determine the corresponding ThunarVfsPath */
-      path = thunar_vfs_path_new (tap_provider->child_watch_path, NULL);
-      if (G_LIKELY (path != NULL))
-        {
-          /* schedule a changed notification on the path */
-          monitor = thunar_vfs_monitor_get_default ();
-          thunar_vfs_monitor_feed (monitor, THUNAR_VFS_MONITOR_EVENT_CHANGED, path);
-          g_object_unref (G_OBJECT (monitor));
-
-          /* release the ThunarVfsPath */
-          thunar_vfs_path_unref (path);
-        }
-    }
-
   /* need to cleanup */
   g_spawn_close_pid (pid);
 
@@ -712,9 +695,7 @@ tap_provider_child_watch_destroy (gpointer user_data)
 {
   TapProvider *tap_provider = TAP_PROVIDER (user_data);
 
-  /* reset child watch id and path */
-  g_free (tap_provider->child_watch_path);
-  tap_provider->child_watch_path = NULL;
+  /* reset child watch id */
   tap_provider->child_watch_id = 0;
 }
 
